@@ -35,6 +35,7 @@ the effective destination project.
 '''
 
 import os, sys, json, subprocess, tempfile, multiprocessing
+import datetime
 import dxpy
 from dxpy import logger
 
@@ -186,7 +187,7 @@ def _inline_documentation_files(app_spec, src_dir):
                     app_spec['developerNotes'] = fh.read()
                 break
 
-def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overwrite=False, project=None, override_folder=None, override_name=None, dx_toolkit_autodep="stable", dry_run=False):
+def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overwrite=False, archive=False, project=None, override_folder=None, override_name=None, dx_toolkit_autodep="stable", dry_run=False):
     """
     Creates a new applet object.
 
@@ -224,6 +225,7 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
     if 'dxapi' not in applet_spec:
         applet_spec['dxapi'] = dxpy.API_VERSION
 
+    archived_applet = None
     if check_name_collisions and not dry_run:
         destination_path = applet_spec['folder'] + ('/' if not applet_spec['folder'].endswith('/') else '') + applet_spec['name']
         logger.debug("Checking for existing applet at " + destination_path)
@@ -232,8 +234,22 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
                 logger.info("Deleting applet %s" % (result['id']))
                 # TODO: test me
                 dxpy.DXProject(dest_project).remove_objects([result['id']])
+            elif archive:
+                logger.info("Archiving applet %s" % (result['id']))
+                proj = dxpy.DXProject(dest_project)
+                archive_folder = '/.Applet_archive'
+                try:
+                    proj.list_folder(archive_folder)
+                except dxpy.DXAPIError:
+                    proj.new_folder(archive_folder)
+
+                proj.move(objects=[result['id']], destination=archive_folder)
+                archived_applet = dxpy.DXApplet(result['id'])
+                now = datetime.datetime.fromtimestamp(archived_applet.created/1000).ctime()
+                new_name = archived_applet.name + " ({d})".format(d=now)
+                archived_applet.rename(new_name)
             else:
-                raise AppBuilderException("An applet already exists at %s (id %s) and the --overwrite (-f) option was not given" % (destination_path, result['id']))
+                raise AppBuilderException("An applet already exists at %s (id %s) and the --overwrite (-f) or --archive (-a) options were not given" % (destination_path, result['id']))
 
     # -----
     # Override various fields from the pristine dxapp.json
@@ -300,6 +316,9 @@ def upload_applet(src_dir, uploaded_resources, check_name_collisions=True, overw
     if "categories" in applet_spec:
         dxpy.DXApplet(applet_id, project=dest_project).add_tags(applet_spec["categories"])
 
+    if archived_applet:
+        archived_applet.set_properties({'replacedWith': archived_applet.get_id()})
+
     return applet_id, applet_spec
 
 def _create_or_update_version(app_name, version, app_spec, try_update=True):
